//
//  FluffView.swift
//  Runner
//
//  Created by Grigori on 7/14/20.
//

import UIKit
import Flutter

public class EmojiView: NSObject, FlutterPlatformView {
    let channel: FlutterMethodChannel

    let frame: CGRect
    let viewId: Int64
    let args: Any?

    var isChangable = true

    init(_ frame: CGRect, viewId: Int64, args: Any?, channel: FlutterMethodChannel){
        self.frame = frame
        self.viewId = viewId
        self.args = args
        self.channel = channel
    }

    public func view() -> UIView {
        var text: String?
        var placeholder: String?

        var fontSize: Int?
        var font: UIFont?
        var tintColor: UIColor?

        if let args = args as? [String: Any]{
            text = args["text"] as? String
            placeholder = args["placeholder"] as? String
            fontSize = args["fontSize"] as? Int

            if let tintColorString = args["tintColor"] as? String{
                tintColor = UIColor.init(hex: tintColorString)
            }

            if let isChangable = args["isChangable"] as? Bool{
                self.isChangable = isChangable
            }

        }


        if let fontSize = fontSize{
            font = UIFont.systemFont(ofSize: CGFloat(fontSize))
        }

        return create(text: text, placeholder: placeholder, font: font, tintColor: tintColor)
    }

    private func create(text: String?, placeholder: String?, font: UIFont?, tintColor: UIColor?) -> UIView{
        let textField = EmojiTextField(frame: frame)
        textField.text = text
        textField.placeholder = placeholder

        textField.font = font
        textField.delegate = self
        textField.tintColor = tintColor

        textField.addTarget(self, action: #selector(textFieldActive), for: UIControl.Event.editingDidBegin)

        return textField
    }
}

private extension EmojiView{
    @objc func textFieldActive(){
        channel.invokeMethod("tap", arguments: nil)
    }
}

extension EmojiView: UITextFieldDelegate{
    public func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool{
        channel.invokeMethod("change", arguments: ["text": string])

        return isChangable
    }
}

class EmojiTextField: UITextField {
    // required for iOS 13
    override var textInputContextIdentifier: String? { "" } // return non-nil to show the Emoji keyboard ¯\_(ツ)_/¯

    override var textInputMode: UITextInputMode? {
        for mode in UITextInputMode.activeInputModes {
            if mode.primaryLanguage == "emoji" {
                return mode
            }
        }
        return nil
    }
}

