import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'native_emoji_keyboard.dart';

typedef EmojiFieldTap = void Function();

class EmojiKeyboardText extends StatefulWidget {
  EmojiKeyboardText(
      {this.text,
      this.fontSize,
      this.placeholder,
      this.tintColor,
      this.isChangable,
      this.onTextChange,
      this.onTap});

  /// Default text
  final String text;

  /// Font size
  final int fontSize;

  /// Text placeholder
  final String placeholder;

  /// Hex tint color
  final String tintColor;

  /// Update UITextfield on write in keyboard
  final bool isChangable;

  /// Called on text change
  final ValueNotifier<String> onTextChange;

  /// Called on tap text field
  final EmojiFieldTap onTap;

  Map _createMap() {
    Map<String, dynamic> map = Map<String, dynamic>();

    map["text"] = text;
    map["fontSize"] = fontSize;
    map["placeholder"] = placeholder;
    map["tintColor"] = tintColor;
    map["isChangable"] = isChangable;

    return map;
  }

  @override
  State<StatefulWidget> createState() {
    return _EmojiKeyboardTextState();
  }
}

class _EmojiKeyboardTextState extends State<EmojiKeyboardText> {
  static const StandardMessageCodec _decoder = StandardMessageCodec();

  @override
  void initState() {
    super.initState();

    addListeners();
  }

  @override
  Widget build(Object context) {
    return UiKitView(
        viewType: 'EmojiTextField',
        creationParams: widget._createMap(),
        creationParamsCodec: _decoder);
  }
}

extension Listeners on _EmojiKeyboardTextState {
  void addListeners() {
    EmojiKeyboard.listForUpdate((String method, dynamic value) {
      if (method == 'change') {
        widget.onTextChange.value = value;
      }

      if (method == 'tap') {
        if (widget.onTap != null) {
          widget.onTap();
        }
      }
    });
  }
}
